Stan Math Library: stan/math/prim/scal/prob/beta_cdf.hpp Source File

| Stan Math Library 2.10.0 reverse mode automatic differentiation

- Main Page
- Modules
- Namespaces
- Classes
- Files

- File List
- File Members

beta_cdf.hpp
Go to the documentation of this file.
1 #ifndef STAN_MATH_PRIM_SCAL_PROB_BETA_CDF_HPP
2 #define STAN_MATH_PRIM_SCAL_PROB_BETA_CDF_HPP
3
4 #include <stan/math/prim/scal/meta/is_constant_struct.hpp>
5 #include <stan/math/prim/scal/meta/OperandsAndPartials.hpp>
6 #include <stan/math/prim/scal/meta/VectorBuilder.hpp>
7 #include <stan/math/prim/scal/meta/include_summand.hpp>
8 #include <stan/math/prim/scal/meta/partials_return_type.hpp>
9 #include <stan/math/prim/scal/meta/contains_nonconstant_struct.hpp>
10 #include <stan/math/prim/scal/err/check_consistent_sizes.hpp>
11 #include <stan/math/prim/scal/err/check_less_or_equal.hpp>
12 #include <stan/math/prim/scal/err/check_nonnegative.hpp>
13 #include <stan/math/prim/scal/err/check_not_nan.hpp>
14 #include <stan/math/prim/scal/err/check_positive_finite.hpp>
15 #include <stan/math/prim/scal/fun/log1m.hpp>
16 #include <stan/math/prim/scal/fun/multiply_log.hpp>
17 #include <stan/math/prim/scal/fun/value_of.hpp>
18 #include <stan/math/prim/scal/fun/digamma.hpp>
19 #include <stan/math/prim/scal/fun/inc_beta.hpp>
20 #include <stan/math/prim/scal/fun/inc_beta_dda.hpp>
21 #include <stan/math/prim/scal/fun/inc_beta_ddb.hpp>
22 #include <stan/math/prim/scal/fun/inc_beta_ddz.hpp>
23 #include <stan/math/prim/scal/fun/lgamma.hpp>
24 #include <stan/math/prim/scal/fun/lbeta.hpp>
25 #include <stan/math/prim/scal/fun/constants.hpp>
26 #include <boost/math/special_functions/gamma.hpp>
27 #include <boost/random/gamma_distribution.hpp>
28 #include <boost/random/variate_generator.hpp>
29 #include <cmath>
30
31 namespace stan {
32
33 namespace math {
34
47 template <typename T_y, typename T_scale_succ, typename T_scale_fail>
48 typename return_type<T_y, T_scale_succ, T_scale_fail>::type
49 beta_cdf(const T_y& y, const T_scale_succ& alpha,
50 const T_scale_fail& beta) {
51 typedef typename stan::partials_return_type<T_y, T_scale_succ,
52 T_scale_fail>::type
53 T_partials_return;
54
55 // Size checks
56 if (!(stan::length(y) && stan::length(alpha)
57 && stan::length(beta)))
58 return 1.0;
59
60 // Error checks
61 static const char* function("stan::math::beta_cdf");
62 using boost::math::tools::promote_args;
63
64 T_partials_return P(1.0);
65
66 check_positive_finite(function, "First shape parameter", alpha);
67 check_positive_finite(function, "Second shape parameter", beta);
68 check_not_nan(function, "Random variable", y);
69 check_consistent_sizes(function,
70 "Random variable", y,
71 "First shape parameter", alpha,
72 "Second shape parameter", beta);
73 check_nonnegative(function, "Random variable", y);
74 check_less_or_equal(function, "Random variable", y, 1);
75
76 // Wrap arguments in vectors
77 VectorView<const T_y> y_vec(y);
78 VectorView<const T_scale_succ> alpha_vec(alpha);
79 VectorView<const T_scale_fail> beta_vec(beta);
80 size_t N = max_size(y, alpha, beta);
81
82 OperandsAndPartials<T_y, T_scale_succ, T_scale_fail>
83 operands_and_partials(y, alpha, beta);
84
85 // Explicit return for extreme values
86 // The gradients are technically ill-defined, but treated as zero
87 for (size_t i = 0; i < stan::length(y); i++) {
88 if (value_of(y_vec[i]) <= 0)
89 return operands_and_partials.value(0.0);
90 }
91
92 // Compute CDF and its gradients
93
94 // Cache a few expensive function calls if alpha or beta is a parameter
95 VectorBuilder<contains_nonconstant_struct<T_scale_succ,
96 T_scale_fail>::value,
97 T_partials_return, T_scale_succ, T_scale_fail>
98 digamma_alpha_vec(max_size(alpha, beta));
99
100 VectorBuilder<contains_nonconstant_struct<T_scale_succ,
101 T_scale_fail>::value,
102 T_partials_return, T_scale_succ, T_scale_fail>
103 digamma_beta_vec(max_size(alpha, beta));
104
105 VectorBuilder<contains_nonconstant_struct<T_scale_succ,
106 T_scale_fail>::value,
107 T_partials_return, T_scale_succ, T_scale_fail>
108 digamma_sum_vec(max_size(alpha, beta));
109
110 if (contains_nonconstant_struct<T_scale_succ, T_scale_fail>::value) {
111 for (size_t n = 0; n < N; n++) {
112 const T_partials_return alpha_dbl = value_of(alpha_vec[n]);
113 const T_partials_return beta_dbl = value_of(beta_vec[n]);
114
115 digamma_alpha_vec[n] = digamma(alpha_dbl);
116 digamma_beta_vec[n] = digamma(beta_dbl);
117 digamma_sum_vec[n] = digamma(alpha_dbl + beta_dbl);
118 }
119 }
120
121 // Compute vectorized CDF and gradient
122 for (size_t n = 0; n < N; n++) {
123 // Explicit results for extreme values
124 // The gradients are technically ill-defined, but treated as zero
125 if (value_of(y_vec[n]) >= 1.0) continue;
126
127 // Pull out values
128 const T_partials_return y_dbl = value_of(y_vec[n]);
129 const T_partials_return alpha_dbl = value_of(alpha_vec[n]);
130 const T_partials_return beta_dbl = value_of(beta_vec[n]);
131
132 // Compute
133 const T_partials_return Pn = inc_beta(alpha_dbl, beta_dbl, y_dbl);
134
135 P *= Pn;
136
137 if (!is_constant_struct<T_y>::value)
138 operands_and_partials.d_x1[n]
139 += inc_beta_ddz(alpha_dbl, beta_dbl, y_dbl) / Pn;
140
141 if (!is_constant_struct<T_scale_succ>::value)
142 operands_and_partials.d_x2[n]
143 += inc_beta_dda(alpha_dbl, beta_dbl, y_dbl,
144 digamma_alpha_vec[n], digamma_sum_vec[n]) / Pn;
145 if (!is_constant_struct<T_scale_fail>::value)
146 operands_and_partials.d_x3[n]
147 += inc_beta_ddb(alpha_dbl, beta_dbl, y_dbl,
148 digamma_beta_vec[n], digamma_sum_vec[n]) / Pn;
149 }
150
151 if (!is_constant_struct<T_y>::value) {
152 for (size_t n = 0; n < stan::length(y); ++n)
153 operands_and_partials.d_x1[n] *= P;
154 }
155 if (!is_constant_struct<T_scale_succ>::value) {
156 for (size_t n = 0; n < stan::length(alpha); ++n)
157 operands_and_partials.d_x2[n] *= P;
158 }
159 if (!is_constant_struct<T_scale_fail>::value) {
160 for (size_t n = 0; n < stan::length(beta); ++n)
161 operands_and_partials.d_x3[n] *= P;
162 }
163
164 return operands_and_partials.value(P);
165 }
166
167 }
168 }
169 #endif
check_not_nan.hpp
stan::math::OperandsAndPartials::d_x2
VectorView< T_return_type, false, true > d_x2
Definition: OperandsAndPartials.hpp:40
lgamma.hpp
stan::math::check_not_nan
bool check_not_nan(const char *function, const char *name, const T_y &y)
Return true if y is not NaN.
Definition: check_not_nan.hpp:63
check_positive_finite.hpp
stan
Definition: log_sum_exp.hpp:8
stan::math::value_of
T value_of(const fvar< T > &v)
Return the value of the specified variable.
Definition: value_of.hpp:16
stan::math::inc_beta_dda
T inc_beta_dda(T a, T b, T z, T digamma_a, T digamma_ab)
Returns the partial derivative of the regularized incomplete beta function, I_{z}(a, b) with respect to a.
Definition: inc_beta_dda.hpp:39
inc_beta_ddz.hpp
digamma.hpp
stan::math::OperandsAndPartials::value
T_return_type value(double value)
Returns a T_return_type with the value specified with the partial derivatves.
Definition: OperandsAndPartials.hpp:68
contains_nonconstant_struct.hpp
stan::length
size_t length(const std::vector< T > &x)
Definition: length.hpp:10
is_constant_struct.hpp
stan::math::inc_beta_ddb
T inc_beta_ddb(T a, T b, T z, T digamma_b, T digamma_ab)
Returns the partial derivative of the regularized incomplete beta function, I_{z}(a, b) with respect to b.
Definition: inc_beta_ddb.hpp:39
stan::math::inc_beta_ddz
T inc_beta_ddz(T a, T b, T z)
Returns the partial derivative of the regularized incomplete beta function, I_{z}(a, b) with respect to z.
Definition: inc_beta_ddz.hpp:27
inc_beta_ddb.hpp
include_summand.hpp
multiply_log.hpp
check_nonnegative.hpp
stan::is_constant_struct
Metaprogram to determine if a type has a base scalar type that can be assigned to type double...
Definition: is_constant_struct.hpp:13
stan::math::inc_beta
fvar< T > inc_beta(const fvar< T > &a, const fvar< T > &b, const fvar< T > &x)
Definition: inc_beta.hpp:20
stan::partials_return_type
Definition: partials_return_type.hpp:16
check_less_or_equal.hpp
stan::math::beta_cdf
return_type< T_y, T_scale_succ, T_scale_fail >::type beta_cdf(const T_y &y, const T_scale_succ &alpha, const T_scale_fail &beta)
Calculates the beta cumulative distribution function for the given variate and scale variables...
Definition: beta_cdf.hpp:49
stan::math::OperandsAndPartials
This class builds partial derivatives with respect to a set of operands.
Definition: OperandsAndPartials.hpp:38
stan::math::OperandsAndPartials::d_x3
VectorView< T_return_type, false, true > d_x3
Definition: OperandsAndPartials.hpp:41
stan::max_size
size_t max_size(const T1 &x1, const T2 &x2)
Definition: max_size.hpp:9
VectorBuilder.hpp
log1m.hpp
partials_return_type.hpp
OperandsAndPartials.hpp
stan::math::check_less_or_equal
bool check_less_or_equal(const char *function, const char *name, const T_y &y, const T_high &high)
Return true if y is less or equal to high.
Definition: check_less_or_equal.hpp:81
stan::contains_nonconstant_struct
Definition: contains_nonconstant_struct.hpp:14
stan::VectorBuilder
VectorBuilder allocates type T1 values to be used as intermediate values.
Definition: VectorBuilder.hpp:28
constants.hpp
stan::math::check_consistent_sizes
bool check_consistent_sizes(const char *function, const char *name1, const T1 &x1, const char *name2, const T2 &x2)
Return true if the dimension of x1 is consistent with x2.
Definition: check_consistent_sizes.hpp:31
lbeta.hpp
inc_beta.hpp
stan::math::check_nonnegative
bool check_nonnegative(const char *function, const char *name, const T_y &y)
Return true if y is non-negative.
Definition: check_nonnegative.hpp:66
value_of.hpp
stan::VectorView
VectorView is a template expression that is constructed with a container or scalar, which it then allows to be used as an array using operator[].
Definition: VectorView.hpp:48
check_consistent_sizes.hpp
stan::math::check_positive_finite
bool check_positive_finite(const char *function, const char *name, const T_y &y)
Return true if y is positive and finite.
Definition: check_positive_finite.hpp:28
stan::math::OperandsAndPartials::d_x1
VectorView< T_return_type, false, true > d_x1
Definition: OperandsAndPartials.hpp:39
stan::math::digamma
fvar< T > digamma(const fvar< T > &x)
Definition: digamma.hpp:16
inc_beta_dda.hpp

[ Stan Home Page ] © 2011–2016, Stan Development Team.